package com.altinity.opsview.odw.bc;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Document;
import org.dom4j.Element;

import com.altinity.opsview.odw.CommandLineParams;
import com.altinity.opsview.odw.elements.WeeklyServiceManagementSummary;
import com.altinity.opsview.odw.elements.WeeklyServiceDetail;
import com.altinity.opsview.odw.elements.WeeklyServiceHelper;
import com.altinity.opsview.odw.elements.WeeklyServiceSummary;
import com.altinity.opsview.odw.elements.WeeklyServiceSummaryTable;
import com.altinity.opsview.odw.exception.InvalidUsageException;
import com.altinity.opsview.odw.exception.ReportException;
import com.altinity.opsview.odw.model.ServiceCheck;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.reports.ReportCriteria;
import com.excira.utils.DateUtils;
import com.excira.utils.StringUtils;

public class WeeklyServiceAvailabilityReportBC extends BaseReportBC  {
	
	/**
	 * 
	 */
	public void init(CommandLineParams params) throws InvalidUsageException {
		
		super.init(params);
		
		// Determine output file path
		// If not output file path has been specified, use the default
		// reports directory
		String currentDir = System.getProperty("user.dir");
		String specifiedPath = params.getOutputFilePath();
		if ( StringUtils.isNullOrEmptyString(specifiedPath) ) {
			
			StringBuffer filePath = new StringBuffer();
			filePath.append(currentDir + "/reports/");
			filePath.append("weekly_service_availability-");
			SimpleDateFormat format = new SimpleDateFormat("dd-MMMM-yyyy");
			filePath.append(format.format(startDate)); 
			filePath.append("-");
			format.applyPattern("yyyy-MM-dd");
			filePath.append(format.format(Calendar.getInstance().getTime()));
			filePath.append(".pdf");		
			
			outputFilePath = filePath.toString();
		}
		// Output file path has been specified
		else {
		
			// Use absolute path specified on command line
			if ( specifiedPath.startsWith("/") ) 
				outputFilePath = specifiedPath;
			
			// Use relative path specified on command line
			else 
				outputFilePath = currentDir + "/" + specifiedPath;
			
			// Check that specified path exists and is read/writable
			File outputDir = new File(StringUtils.substringBeforeLast(outputFilePath, "/"));
			if ( !outputDir.exists() || !outputDir.canRead() || !outputDir.canWrite() )
				throw new InvalidUsageException();
		}
	}
	
	/**
	 * 
	 */
	public void createReport() throws ReportException {
		
		// First get the keyed servicechecks
		WeeklyServiceHelper helper = new WeeklyServiceHelper(reportCriteria, dao, configDao);
		List <ServiceCheckGroup> keyedSCs = helper.getServiceChecksByText();
		
		// First set up each module and make sure we have the data
		// to generate report
		WeeklyServiceSummary summary =
			new WeeklyServiceSummary(reportCriteria, dao, configDao, log, keyedSCs);
		summary.runQuery();
		
		WeeklyServiceSummaryTable table = 
			new WeeklyServiceSummaryTable(reportCriteria, dao, configDao, log, keyedSCs);
		table.runQuery();
		
		WeeklyServiceManagementSummary manSummary =
			new WeeklyServiceManagementSummary(reportCriteria, dao, configDao, log);
		manSummary.runQuery();
		
		WeeklyServiceDetail detail =
			new WeeklyServiceDetail(reportCriteria, dao, configDao, log, keyedSCs);
		detail.runQuery();		
		
		Element reportSections = getReportSections();
		
		// Create summary section
		Element summarySectionEl = reportSections.addElement("section");
		summarySectionEl.addElement("title").setText("Availability Summary");
		summarySectionEl.addAttribute("name", "availabilitySummary");
		
		// Add elements to summary
		Element summaryElementsEl = summarySectionEl.addElement("elements");
		summary.addElement(summaryElementsEl);
		
		// Create summary table section
		Element tableSectionEl = reportSections.addElement("section");
		tableSectionEl.addElement("title").setText("Availability Summary");
		tableSectionEl.addAttribute("name", "availabilitySummaryTable");
		tableSectionEl.addAttribute("section-format", "report" + reportCriteria.getFormat() + "Portrait");
		
		// Add elements to table
		Element tableElementsEl = tableSectionEl.addElement("elements");
		table.addElement(tableElementsEl);
		
		// Create management section if necessary
		if ( manSummary.hasAnnotation() ) {
			
			Element manSectionEl = reportSections.addElement("section");
			manSectionEl.addElement("title").setText("Management Summary");
			manSectionEl.addAttribute("name", "managementSummary");
			
			// Add elements to management summary
			Element manElementsEl = manSectionEl.addElement("elements");
			manSummary.addElement(manElementsEl);
		}
		
		// Create detail section
		Element detailSectionEl = reportSections.addElement("section");
		detailSectionEl.addElement("title").setText("Availability Detail");
		detailSectionEl.addAttribute("name", "availabilityDetail");
	
		// Add elements to detail
		Element detailElementsEl = detailSectionEl.addElement("elements");
		detail.addElement(detailElementsEl);
		
		// If there are no errors at this point, the report has 
		// been genrated and the host group availability is 
		// the last section
		if ( getLog().isClean() )
			detailSectionEl.addAttribute("lastSection", "true");		
	}
	
	/**
	 * 
	 */
	protected Date initStartDate(CommandLineParams params) throws InvalidUsageException {
		
		String dateParam = params.getStrStartDate();
		
		// First confirm that start date is valid. Must be yyyy-MM-dd or -n
		if ( !DateUtils.isValidDate(dateParam, CommandLineParams.YYYY_MM_DD_REGX, "yyyy-MM-dd")
				&& !StringUtils.matchesPattern(dateParam, CommandLineParams.N_REGX) )
			throw new InvalidUsageException();
		
		Calendar cal = Calendar.getInstance();
		DateUtils.clearHourMinuteSecond(cal);
		
		// Check if a start date has been specified
		if ( StringUtils.notNullOrEmpty(dateParam) ) {
			 
			// Handle case where -n has been specified
			if ( StringUtils.matchesPattern(dateParam, CommandLineParams.N_REGX) ) {
				
				// Update week field of calendar and get start date
				cal.add(Calendar.WEEK_OF_YEAR, new Integer(dateParam));
				
			}
			// Otherwise yyyy-MM-dd has been specified
			else {
			
				cal.setTime(DateUtils.parse(dateParam, "yyyy-MM-dd"));
			}
		}
		
		// Finish calendar setup and return date
		DateUtils.adjustDateAccordingToWeekConfigToGetFirstDayOfWeek(cal, params.getStartOfWeek() + 1); // Add 1 as Sunday = 1 in Java, Sunday = 0 in ODW

		return cal.getTime();
	}
	
	/**
	 * 
	 */
	protected Date initEndDate(CommandLineParams params) {
	
		// Get the start date calendar configured first
		Calendar startCal = Calendar.getInstance();
		startCal.setTime(startDate);
		DateUtils.adjustDateAccordingToWeekConfigToGetFirstDayOfWeek(startCal, params.getStartOfWeek());
		
		// Now add one week
		startCal.add(Calendar.DAY_OF_WEEK, 7);
		
		// Finally, add the hours, minutes, seconds and return the date
		DateUtils.setMaxHourMinuteSecond(startCal);
		return startCal.getTime();
	}
	
	/**
	 * 
	 */
	public Map <String, ServiceCheckGroup> groupServiceChecks(List <ServiceCheck> serviceChecks) {

		// Generate map of servicecheck hostname/name key to servicecheck ids
		Map <String,ServiceCheckGroup> keyedSCs = new HashMap<String, ServiceCheckGroup>();
		for ( ServiceCheck sc : serviceChecks ) {
			
			// Get the hostname/name key for this servicecheck
			String key = sc.getName() + sc.getHostName();
			
			// Get the corresponding list for this key
			ServiceCheckGroup group = keyedSCs.get(key);
			
			// Create the list if it doesn't
			// already exist
			if ( group == null ) {
				
				group = new ServiceCheckGroup();
				keyedSCs.put(key, group);
			}
			
			// Add servicecheck to group
			group.addServiceCheck(sc);
		}
		
		return keyedSCs;
	}
	
	
	/**
	 * Called from JUnit
	 */
	public void setReportCriteria(ReportCriteria rc) {
		
		this.reportCriteria = rc;
	}
	
	/**
	 * Called from JUnit
	 */
	public void setDocument(Document doc) {
		
		this.doc = doc;
	}
	
	/**
	 * Called from JUnit
	 */
	public void setLog(ODWReportLogger log) {
		
		this.log = log;
	}	
}
