package com.altinity.opsview.odw.elements;

import java.text.DecimalFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.dom4j.Element;

import com.altinity.opsview.odw.ReportConstants;
import com.altinity.opsview.odw.bc.ODWReportLogger;
import com.altinity.opsview.odw.dao.ODWReportConfigDAO;
import com.altinity.opsview.odw.dao.ODWReportDAO;
import com.altinity.opsview.odw.model.ServiceCheckGroup;
import com.altinity.opsview.odw.model.WeeklyServiceAvailability;
import com.altinity.opsview.odw.reports.ReportCriteria;

public class WeeklyServiceSummaryTable extends AbstractReportElement implements ReportConstants {
	
	protected List <ServiceCheckGroup> serviceCheckGroups;
	protected Map <String, WeeklyServiceAvailability> availByText;
	protected boolean useIndex;
	
	/**
	 *
	 */
	public WeeklyServiceSummaryTable(ReportCriteria rc, ODWReportDAO dao, ODWReportConfigDAO configDao, 
			ODWReportLogger log, List <ServiceCheckGroup> groups) {
		super(rc, dao, configDao, log);
		
		this.serviceCheckGroups = groups;
	}

	/**
	 * 
	 */
	public Element addElement(Element elements) {
		
		Element summaryEl = elements.addElement("element");
		summaryEl.addElement("title").setText("Weekly Availability Summary Table");
		summaryEl.addAttribute("name", "weeklySummaryTable");
		
		// Handle case where data has been found
		if ( !availByText.isEmpty() ) {
			
			addPercentageTable(summaryEl, "AVAILABILITY PERCENTAGE");
		}
		// Otherwise no data has been found
		else {
			
			// Add empty element
			summaryEl.addElement("empty").setText("true");
			
			// Report error
			log.getErrors().add("No data found for module 'Weekly Availability Summary Table'");
		}
		
		return summaryEl;
	}
	
	/**
	 * 
	 */
	public void runQuery() {
		
		// We need to determine if these servicecheck labels
		// are to prepended with an index
		setUseIndex(serviceCheckGroups);
		
		availByText = new LinkedHashMap<String, WeeklyServiceAvailability>();
		
		// Now query availability for each servicecheck
		int index = 1;
		for ( ServiceCheckGroup group : serviceCheckGroups ) {
			
			// Add the availability for this servicecheck to the list
			WeeklyServiceAvailability summary = dao.getWeeklyAvailabilityForServiceCheck(rc, group.getIds());
			if ( summary.isEmpty() )
				continue;
			
			String text = group.getText();
			if ( useIndex )
				text = index + ": " + text;
			
			// Now add the availability to the map using text as key
			availByText.put(group.getText(), summary);
			
			index++;
		}
	}
	
	/**
	 * 
	 */
	private void setUseIndex(List <ServiceCheckGroup> groups) {
		
		int maxLength = 0;
		for ( ServiceCheckGroup group : groups ) {
			
			int length = group.getText().length();
			if ( length > maxLength )
				maxLength = length;

		}
		
		// Chart will only show labels of length less than 28
		// If the label is longer than 28 characters, we want to
		// prepend the label with an index
		useIndex = maxLength >= 28;
	}	
	
	/**
	 * 
	 */
	protected void addPercentageTable(Element parentEl, String title) {
		
		Element tableEl = parentEl.addElement("availPercentage");
		tableEl.addElement("title").setText(title);
		if ( useIndex )
			tableEl.addElement("index").setText("true");
		
		DecimalFormat percentFormat = new DecimalFormat(PERCENT_FORMAT);
		
		Element serviceChecksEl = tableEl.addElement("servicechecks");
		
		int index = 1;
		for ( String text : availByText.keySet() ) {
			
			WeeklyServiceAvailability avail = availByText.get(text);
			
			Element serviceCheckEl = serviceChecksEl.addElement("servicecheck");
			if ( useIndex )
				serviceCheckEl.addElement("index").setText("" + index);
			serviceCheckEl.addElement("title").setText(text);
			serviceCheckEl.addElement("ok").setText(percentFormat.format(avail.getPercentOk()));
			serviceCheckEl.addElement("sched").setText(percentFormat.format(avail.getPercentScheduled()));
			serviceCheckEl.addElement("unsched").setText(percentFormat.format(avail.getPercentUnscheduled()));
			
			index++;
		}
	}
}
